@@ -1,4 +1,4 @@
-## identify.phylo.R (2008-01-14)
+## identify.phylo.R (2008-02-28)
 
 ##   Graphical Identification of Nodes and Tips
 
@@ -12,22 +12,21 @@ identify.phylo <- function(x, nodes = TRUE, tips = FALSE,
 {
     cat("Click close to a node of the tree...\n")
     xy <- locator(1)
-    Ntip <- .last_plot.phylo$Ntip
-    d <- sqrt((xy$x - .last_plot.phylo$xx)^2 +
-              (xy$y - .last_plot.phylo$yy)^2)
+    lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
+    d <- sqrt((xy$x - lastPP$xx)^2 + (xy$y - lastPP$yy)^2)
     NODE <- which.min(d)
     res <- list()
-    if (NODE <= Ntip) {
+    if (NODE <= lastPP$Ntip) {
         res$tips <- if (labels) x$tip.label[NODE] else NODE
         return(res)
     }
     if (tips) {
-        TIPS <- prop.part(x)[[NODE - Ntip]]
+        TIPS <- prop.part(x)[[NODE - lastPP$Ntip]]
         res$tips <- if (labels) x$tip.label[TIPS] else TIPS
     }
     if (nodes) {
         if (is.null(x$node.label)) labels <- FALSE
-        res$nodes <- if (labels) x$node.label[NODE - Ntip] else NODE
+        res$nodes <- if (labels) x$node.label[NODE - lastPP$Ntip] else NODE
     }
     res
 }
